@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #pragma once
 
 // Test these using #if AT_CUDNN_ENABLED(), not #ifdef, so that it's
@@ -6,10 +7,11 @@
 //
 // NB: This header MUST NOT be included from other headers; it should
 // only be included from C++ files.
-
 #define AT_CUDNN_ENABLED() 0
+#define AT_CUSPARSELT_ENABLED() 0
+#define AT_HIPSPARSELT_ENABLED() 0
 #define AT_ROCM_ENABLED() 1
-#define AT_MAGMA_ENABLED() 0
+#define AT_MAGMA_ENABLED() 1
 
 // Needed for hipMAGMA to correctly identify implementation
 #if (AT_ROCM_ENABLED() && AT_MAGMA_ENABLED())
@@ -17,3 +19,7 @@
 #endif
 
 #define NVCC_FLAGS_EXTRA ""
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
