@@ -1,3 +1,4 @@
+#if !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
 #ifndef THCP_STREAM_INC
 #define THCP_STREAM_INC
 
@@ -18,3 +19,7 @@ inline bool THCPStream_Check(PyObject* obj) {
 }
 
 #endif // THCP_STREAM_INC
+
+#else
+#error "This file should not be included when either TORCH_STABLE_ONLY or TORCH_TARGET_VERSION is defined."
+#endif  // !defined(TORCH_STABLE_ONLY) && !defined(TORCH_TARGET_VERSION)
